@@ -1,4 +1,10 @@
-use jack_sys as j;
+#[cfg(feature = "dlopen")]
+use crate::LIB;
+#[cfg(all(feature = "dlopen", feature = "metadata"))]
+use crate::UUID;
+use dlib::ffi_dispatch;
+#[cfg(not(feature = "dlopen"))]
+use jack_sys::*;
 use std::fmt::Debug;
 use std::sync::Arc;
 use std::{ffi, fmt, ptr};
@@ -28,13 +34,14 @@ use crate::{
 /// };
 /// ```
 
-pub type InternalClientID = j::jack_intclient_t;
+pub type InternalClientID = jack_sys::jack_intclient_t;
 
-pub struct Client(
-    *mut j::jack_client_t,
-    Arc<()>,
-    Option<Box<dyn PropertyChangeHandler>>,
-);
+#[allow(dead_code)]
+pub struct Client {
+    inner: *mut jack_sys::jack_client_t,
+    life: Arc<()>,
+    property_change_handler: Option<Box<dyn PropertyChangeHandler>>,
+}
 
 unsafe impl Send for Client {}
 unsafe impl Sync for Client {}
@@ -48,19 +55,50 @@ impl Client {
     /// errors when attempting to opening. To access these, check the returned `ClientStatus`.
     pub fn new(client_name: &str, options: ClientOptions) -> Result<(Self, ClientStatus), Error> {
         let _m = CREATE_OR_DESTROY_CLIENT_MUTEX.lock().unwrap();
-        crate::logging::initialize_logging();
+        #[cfg(feature = "dlopen")]
+        let lib = crate::LIB_RESULT
+            .as_ref()
+            .map_err(|e| Error::LoadLibraryError(format!("{}", e)))?;
+        unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                lib,
+                jack_set_error_function,
+                Some(error_fn)
+            );
+            ffi_dispatch!(
+                feature = "dlopen",
+                lib,
+                jack_set_info_function,
+                Some(info_fn)
+            );
+        }
         sleep_on_test();
         let mut status_bits = 0;
         let client = unsafe {
             let client_name = ffi::CString::new(client_name).unwrap();
-            j::jack_client_open(client_name.as_ptr(), options.bits(), &mut status_bits)
+            ffi_dispatch!(
+                feature = "dlopen",
+                lib,
+                jack_client_open,
+                client_name.as_ptr(),
+                options.bits(),
+                &mut status_bits
+            )
         };
         sleep_on_test();
         let status = ClientStatus::from_bits(status_bits).unwrap_or_else(ClientStatus::empty);
         if client.is_null() {
             Err(Error::ClientError(status))
         } else {
-            Ok((Client(client, Arc::default(), None), status))
+            Ok((
+                Client {
+                    inner: client,
+                    life: Arc::default(),
+                    property_change_handler: None,
+                },
+                status,
+            ))
         }
     }
 
@@ -81,7 +119,8 @@ impl Client {
     /// The sample rate of the JACK system, as set by the user when jackd was
     /// started.
     pub fn sample_rate(&self) -> usize {
-        let srate = unsafe { j::jack_get_sample_rate(self.raw()) };
+        let srate =
+            unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_get_sample_rate, self.raw()) };
         srate as usize
     }
 
@@ -91,7 +130,7 @@ impl Client {
     /// clients as a percentage of the real time available per cycle determined by the buffer size
     /// and sample rate.
     pub fn cpu_load(&self) -> f32 {
-        let load = unsafe { j::jack_cpu_load(self.raw()) };
+        let load = unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_cpu_load, self.raw()) };
         load as f32
     }
 
@@ -101,7 +140,7 @@ impl Client {
     /// `NAME_NOT_UNIQUE`.
     pub fn name(&self) -> &str {
         unsafe {
-            let ptr = j::jack_get_client_name(self.raw());
+            let ptr = ffi_dispatch!(feature = "dlopen", LIB, jack_get_client_name, self.raw());
             let cstr = ffi::CStr::from_ptr(ptr);
             cstr.to_str().unwrap()
         }
@@ -110,7 +149,7 @@ impl Client {
     /// The current maximum size that will every be passed to the process
     /// callback.
     pub fn buffer_size(&self) -> Frames {
-        unsafe { j::jack_get_buffer_size(self.raw()) }
+        unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_get_buffer_size, self.raw()) }
     }
 
     /// Change the buffer size passed to the process callback.
@@ -119,7 +158,15 @@ impl Client {
     /// callback functions before restarting the process cycle. This will cause a gap in the audio
     /// flow, so it should only be done at appropriate stopping points.
     pub fn set_buffer_size(&self, n_frames: Frames) -> Result<(), Error> {
-        let res = unsafe { j::jack_set_buffer_size(self.raw(), n_frames) };
+        let res = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_set_buffer_size,
+                self.raw(),
+                n_frames
+            )
+        };
         match res {
             0 => Ok(()),
             _ => Err(Error::SetBufferSizeError),
@@ -132,13 +179,13 @@ impl Client {
     ///
     /// * Deallocates, not realtime safe.
     #[cfg(feature = "metadata")]
-    pub fn uuid(&self) -> j::jack_uuid_t {
+    pub fn uuid(&self) -> jack_sys::jack_uuid_t {
         unsafe {
-            let mut uuid: j::jack_uuid_t = Default::default();
-            let uuid_s = j::jack_client_get_uuid(self.raw());
+            let mut uuid: jack_sys::jack_uuid_t = Default::default();
+            let uuid_s = ffi_dispatch!(feature = "dlopen", LIB, jack_client_get_uuid, self.raw());
             assert!(!uuid_s.is_null());
-            assert_eq!(0, j::jack_uuid_parse(uuid_s, &mut uuid));
-            j::jack_free(uuid_s as _);
+            assert_eq!(0, ffi_dispatch!(UUID, jack_uuid_parse, uuid_s, &mut uuid));
+            ffi_dispatch!(feature = "dlopen", LIB, jack_free, uuid_s as _);
             uuid
         }
     }
@@ -150,20 +197,26 @@ impl Client {
     /// * Allocates & deallocates, not realtime safe.
     pub fn uuid_string(&self) -> String {
         unsafe {
-            let uuid_s = j::jack_client_get_uuid(self.raw());
+            let uuid_s = ffi_dispatch!(feature = "dlopen", LIB, jack_client_get_uuid, self.raw());
             assert!(!uuid_s.is_null());
             let uuid = ffi::CStr::from_ptr(uuid_s)
                 .to_str()
                 .expect("uuid is valid string")
                 .to_string();
-            j::jack_free(uuid_s as _);
+            ffi_dispatch!(feature = "dlopen", LIB, jack_free, uuid_s as _);
             uuid
         }
     }
 
     //helper to get client name from uuid string.
     unsafe fn name_by_uuid_raw(&self, uuid: *const ::libc::c_char) -> Option<String> {
-        let name_ptr = j::jack_get_client_name_by_uuid(self.raw(), uuid);
+        let name_ptr = ffi_dispatch!(
+            feature = "dlopen",
+            LIB,
+            jack_get_client_name_by_uuid,
+            self.raw(),
+            uuid
+        );
         if name_ptr.is_null() {
             None
         } else {
@@ -178,10 +231,11 @@ impl Client {
 
     /// Get the name of a client by its numeric uuid.
     #[cfg(feature = "metadata")]
-    pub fn name_by_uuid(&self, uuid: j::jack_uuid_t) -> Option<String> {
+    pub fn name_by_uuid(&self, uuid: jack_sys::jack_uuid_t) -> Option<String> {
         let mut uuid_s = ['\0' as _; 37]; //jack_uuid_unparse expects an array of length 37
+
         unsafe {
-            j::jack_uuid_unparse(uuid, uuid_s.as_mut_ptr());
+            ffi_dispatch!(UUID, jack_uuid_unparse, uuid, uuid_s.as_mut_ptr());
             self.name_by_uuid_raw(uuid_s.as_ptr())
         }
     }
@@ -213,12 +267,16 @@ impl Client {
         let port_name_pattern_cstr = ffi::CString::new(port_name_pattern.unwrap_or("")).unwrap();
         let type_name_pattern_cstr = ffi::CString::new(type_name_pattern.unwrap_or("")).unwrap();
         let flags = libc::c_ulong::from(flags.bits());
+        let ptr = self.raw();
         unsafe {
-            let ports = j::jack_get_ports(
-                self.raw(),
+            let ports = ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_get_ports,
+                ptr,
                 port_name_pattern_cstr.as_ptr(),
                 type_name_pattern_cstr.as_ptr(),
-                flags,
+                flags
             );
             collect_strs(ports)
         }
@@ -247,39 +305,58 @@ impl Client {
         let port_flags = port_spec.jack_flags().bits();
         let buffer_size = port_spec.jack_buffer_size();
         let pp = unsafe {
-            j::jack_port_register(
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_port_register,
                 self.raw(),
                 port_name_c.as_ptr(),
                 port_type_c.as_ptr(),
                 libc::c_ulong::from(port_flags),
-                buffer_size,
+                buffer_size
             )
         };
         if pp.is_null() {
             Err(Error::PortRegistrationError(port_name.to_string()))
         } else {
-            Ok(unsafe { Port::from_raw(port_spec, self.raw(), pp, Arc::downgrade(&self.1)) })
+            Ok(unsafe { Port::from_raw(port_spec, self.raw(), pp, Arc::downgrade(&self.life)) })
         }
     }
 
     /// Get a `Port` by its port id.
     pub fn port_by_id(&self, port_id: PortId) -> Option<Port<Unowned>> {
-        let pp = unsafe { j::jack_port_by_id(self.raw(), port_id) };
+        let pp = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_port_by_id,
+                self.raw(),
+                port_id
+            )
+        };
         if pp.is_null() {
             None
         } else {
-            Some(unsafe { Port::from_raw(Unowned {}, self.raw(), pp, Arc::downgrade(&self.1)) })
+            Some(unsafe { Port::from_raw(Unowned {}, self.raw(), pp, Arc::downgrade(&self.life)) })
         }
     }
 
     /// Get a `Port` by its port name.
     pub fn port_by_name(&self, port_name: &str) -> Option<Port<Unowned>> {
         let port_name = ffi::CString::new(port_name).unwrap();
-        let pp = unsafe { j::jack_port_by_name(self.raw(), port_name.as_ptr()) };
+        let pp = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_port_by_name,
+                self.raw(),
+                port_name.as_ptr()
+            )
+        };
         if pp.is_null() {
             None
         } else {
-            Some(unsafe { Port::from_raw(Unowned {}, self.raw(), pp, Arc::downgrade(&self.1)) })
+            Some(unsafe { Port::from_raw(Unowned {}, self.raw(), pp, Arc::downgrade(&self.life)) })
         }
     }
 
@@ -307,16 +384,19 @@ impl Client {
         let ffi_client_args = ffi::CString::new(client_args).unwrap();
 
         let mut status_bits = 0;
-        let options: j::Enum_JackOptions = j::JackLoadName | j::JackLoadInit;
+        let options: jack_sys::Enum_JackOptions = jack_sys::JackLoadName | jack_sys::JackLoadInit;
 
         let intclient = unsafe {
-            j::jack_internal_client_load(
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_internal_client_load,
                 self.raw(),
                 ffi_client_name.as_ptr(),
                 options,
                 &mut status_bits,
-                ffi_client_bin.as_ptr(),
-                ffi_client_args.as_ptr(),
+                ffi_client_bin.as_ptr() as _,
+                ffi_client_args.as_ptr()
             )
         };
 
@@ -338,7 +418,13 @@ impl Client {
     /// It returns a ClientError on error.
     pub fn unload_internal_client(&self, client: InternalClientID) -> Result<(), Error> {
         let status = unsafe {
-            let status = j::jack_internal_client_unload(self.raw(), client);
+            let status = ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_internal_client_unload,
+                self.raw(),
+                client
+            );
             ClientStatus::from_bits_unchecked(status)
         };
         if status.is_empty() {
@@ -351,7 +437,14 @@ impl Client {
     /// The estimated time in frames that has passed since the JACK server began the current process
     /// cycle.
     pub fn frames_since_cycle_start(&self) -> Frames {
-        unsafe { j::jack_frames_since_cycle_start(self.raw()) }
+        unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_frames_since_cycle_start,
+                self.raw()
+            )
+        }
     }
 
     /// The estimated current time in frames. This function is intended for use in other threads
@@ -362,7 +455,7 @@ impl Client {
     /// # TODO
     /// - test
     pub fn frame_time(&self) -> Frames {
-        unsafe { j::jack_frame_time(self.raw()) }
+        unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_frame_time, self.raw()) }
     }
 
     /// The estimated time in microseconds of the specified frame time
@@ -370,7 +463,15 @@ impl Client {
     /// # TODO
     /// - Improve test
     pub fn frames_to_time(&self, n_frames: Frames) -> Time {
-        unsafe { j::jack_frames_to_time(self.raw(), n_frames) }
+        unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_frames_to_time,
+                self.raw(),
+                n_frames
+            )
+        }
     }
 
     /// The estimated time in frames for the specified system time.
@@ -378,12 +479,23 @@ impl Client {
     /// # TODO
     /// - Improve test
     pub fn time_to_frames(&self, t: Time) -> Frames {
-        unsafe { j::jack_time_to_frames(self.raw(), t) }
+        unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_time_to_frames, self.raw(), t) }
     }
 
     /// Returns `true` if the port `port` belongs to this client.
     pub fn is_mine<PS: PortSpec>(&self, port: &Port<PS>) -> bool {
-        matches!(unsafe { j::jack_port_is_mine(self.raw(), port.raw()) }, 1)
+        matches!(
+            unsafe {
+                ffi_dispatch!(
+                    feature = "dlopen",
+                    LIB,
+                    jack_port_is_mine,
+                    self.raw(),
+                    port.raw()
+                )
+            },
+            1
+        )
     }
 
     /// Toggle input monitoring for the port with name `port_name`.
@@ -398,10 +510,13 @@ impl Client {
     ) -> Result<(), Error> {
         let port_name_cstr = ffi::CString::new(port_name).unwrap();
         let res = unsafe {
-            j::jack_port_request_monitor_by_name(
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_port_request_monitor_by_name,
                 self.raw(),
                 port_name_cstr.as_ptr(),
-                if enable_monitor { 1 } else { 0 },
+                if enable_monitor { 1 } else { 0 }
             )
         };
         match res {
@@ -429,7 +544,7 @@ impl Client {
     //         true => 0,
     //         false => 1,
     //     };
-    //     match unsafe { j::jack_set_freewheel(self.raw(), onoff) } {
+    //     match unsafe { jack_sys::jack_set_freewheel(self.raw(), onoff) } {
     //         0 => Ok(()),
     //         _ => Err(Error::FreewheelError),
     //     }
@@ -459,8 +574,16 @@ impl Client {
         let source_cstr = ffi::CString::new(source_port).unwrap();
         let destination_cstr = ffi::CString::new(destination_port).unwrap();
 
-        let res =
-            unsafe { j::jack_connect(self.raw(), source_cstr.as_ptr(), destination_cstr.as_ptr()) };
+        let res = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_connect,
+                self.raw(),
+                source_cstr.as_ptr(),
+                destination_cstr.as_ptr()
+            )
+        };
         match res {
             0 => Ok(()),
             ::libc::EEXIST => Err(Error::PortAlreadyConnected(
@@ -497,7 +620,15 @@ impl Client {
 
     /// Remove all connections to/from the port.
     pub fn disconnect<PS>(&self, port: &Port<PS>) -> Result<(), Error> {
-        let res = unsafe { j::jack_port_disconnect(self.raw(), port.raw()) };
+        let res = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_port_disconnect,
+                self.raw(),
+                port.raw()
+            )
+        };
         match res {
             0 => Ok(()),
             _ => Err(Error::PortDisconnectionError),
@@ -505,7 +636,15 @@ impl Client {
     }
 
     pub fn unregister_port<PS>(&self, port: Port<PS>) -> Result<(), Error> {
-        let res = unsafe { j::jack_port_unregister(self.raw(), port.raw()) };
+        let res = unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_port_unregister,
+                self.raw(),
+                port.raw()
+            )
+        };
         match res {
             0 => Ok(()),
             _ => Err(Error::PortDisconnectionError),
@@ -530,7 +669,14 @@ impl Client {
         let source_port = ffi::CString::new(source_port).unwrap();
         let destination_port = ffi::CString::new(destination_port).unwrap();
         let res = unsafe {
-            j::jack_disconnect(self.raw(), source_port.as_ptr(), destination_port.as_ptr())
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_disconnect,
+                self.raw(),
+                source_port.as_ptr(),
+                destination_port.as_ptr()
+            )
         };
         match res {
             0 => Ok(()),
@@ -545,15 +691,21 @@ impl Client {
     /// * This function may only be called in a buffer size callback.
     pub unsafe fn type_buffer_size(&self, port_type: &str) -> usize {
         let port_type = ffi::CString::new(port_type).unwrap();
-        j::jack_port_type_get_buffer_size(self.raw(), port_type.as_ptr())
+        ffi_dispatch!(
+            feature = "dlopen",
+            LIB,
+            jack_port_type_get_buffer_size,
+            self.raw(),
+            port_type.as_ptr()
+        )
     }
 
     /// Expose the underlying ffi pointer.
     ///
     /// This is mostly for use within the jack crate itself.
     #[inline(always)]
-    pub fn raw(&self) -> *mut j::jack_client_t {
-        self.0
+    pub fn raw(&self) -> *mut jack_sys::jack_client_t {
+        self.inner
     }
 
     /// Create a `Client` from an ffi pointer.
@@ -562,8 +714,12 @@ impl Client {
     ///
     /// # Safety
     /// It is unsafe to create a `Client` from a raw pointer.
-    pub unsafe fn from_raw(p: *mut j::jack_client_t) -> Self {
-        Client(p, Arc::default(), None)
+    pub unsafe fn from_raw(p: *mut jack_sys::jack_client_t) -> Self {
+        Client {
+            inner: p,
+            life: Arc::default(),
+            property_change_handler: None,
+        }
     }
 
     /// Get a `Transport` object associated with this client.
@@ -572,8 +728,8 @@ impl Client {
     /// * The transport methods will only work during this client's lifetime.
     pub fn transport(&self) -> Transport {
         Transport {
-            client_ptr: self.0,
-            client_life: Arc::downgrade(&self.1),
+            client_ptr: self.inner,
+            client_life: Arc::downgrade(&self.life),
         }
     }
 
@@ -589,14 +745,17 @@ impl Client {
         &mut self,
         handler: H,
     ) -> Result<(), Error> {
-        assert!(self.2.is_none());
+        assert!(self.property_change_handler.is_none());
         let handler = Box::into_raw(Box::new(handler));
         unsafe {
-            self.2 = Some(Box::from_raw(handler));
-            if j::jack_set_property_change_callback(
+            self.property_change_handler = Some(Box::from_raw(handler));
+            if ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_set_property_change_callback,
                 self.raw(),
                 Some(crate::properties::property_changed::<H>),
-                std::mem::transmute::<_, _>(handler),
+                std::mem::transmute::<_, _>(handler)
             ) == 0
             {
                 Ok(())
@@ -615,10 +774,10 @@ impl Drop for Client {
         debug_assert!(!self.raw().is_null()); // Rep invariant
                                               // Close the client
         sleep_on_test();
-        let res = unsafe { j::jack_client_close(self.raw()) }; // close the client
+        let res = unsafe { ffi_dispatch!(feature = "dlopen", LIB, jack_client_close, self.raw()) }; // close the client
         sleep_on_test();
         assert_eq!(res, 0);
-        self.0 = ptr::null_mut();
+        self.inner = ptr::null_mut();
     }
 }
 
@@ -638,7 +797,7 @@ impl Debug for Client {
 /// callback.
 #[derive(Debug)]
 pub struct ProcessScope {
-    client_ptr: *mut j::jack_client_t,
+    client_ptr: *mut jack_sys::jack_client_t,
 
     // Used to allow safe access to IO port buffers
     n_frames: Frames,
@@ -655,13 +814,27 @@ impl ProcessScope {
     /// from the process callback, and can be used to interpret timestamps generated by
     /// `self.frame_time()` in other threads, with respect to the current process cycle.
     pub fn last_frame_time(&self) -> Frames {
-        unsafe { j::jack_last_frame_time(self.client_ptr()) }
+        unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_last_frame_time,
+                self.client_ptr()
+            )
+        }
     }
 
     /// The estimated time in frames that has passed since the JACK server began the current process
     /// cycle.
     pub fn frames_since_cycle_start(&self) -> Frames {
-        unsafe { j::jack_frames_since_cycle_start(self.client_ptr()) }
+        unsafe {
+            ffi_dispatch!(
+                feature = "dlopen",
+                LIB,
+                jack_frames_since_cycle_start,
+                self.client_ptr()
+            )
+        }
     }
 
     /// Provides the internal cycle timing information as used by most of the other time related
@@ -678,14 +851,14 @@ impl ProcessScope {
         let mut next_usecs: Time = 0;
         let mut period_usecs: libc::c_float = 0.0;
 
-        let jack_get_cycle_times = {
-            match *j::jack_get_cycle_times {
+        let jack_get_cycle_times_fn = {
+            match *jack_sys::jack_get_cycle_times {
                 Some(f) => f,
                 None => return Err(Error::WeakFunctionNotFound),
             }
         };
         let res = unsafe {
-            (jack_get_cycle_times)(
+            (jack_get_cycle_times_fn)(
                 self.client_ptr(),
                 &mut current_frames,
                 &mut current_usecs,
@@ -708,7 +881,7 @@ impl ProcessScope {
     ///
     /// This is mostly for use within the jack crate itself.
     #[inline(always)]
-    pub fn client_ptr(&self) -> *mut j::jack_client_t {
+    pub fn client_ptr(&self) -> *mut jack_sys::jack_client_t {
         self.client_ptr
     }
 
@@ -720,7 +893,7 @@ impl ProcessScope {
     /// # Safety
     /// It is unsafe to create a `ProcessScope` since it may not be valid. For library user's, the
     /// `ProcessScope` is usually passed in as a parameter to a trait's method.
-    pub unsafe fn from_raw(n_frames: Frames, client_ptr: *mut j::jack_client_t) -> Self {
+    pub unsafe fn from_raw(n_frames: Frames, client_ptr: *mut jack_sys::jack_client_t) -> Self {
         ProcessScope {
             n_frames,
             client_ptr,
@@ -736,3 +909,17 @@ pub struct CycleTimes {
     pub next_usecs: Time,
     pub period_usecs: libc::c_float,
 }
+
+unsafe extern "C" fn error_fn(msg: *const libc::c_char) {
+    match std::ffi::CStr::from_ptr(msg).to_str() {
+        Ok(msg) => log::error!("{}", msg),
+        Err(err) => log::error!("failed to parse JACK error: {:?}", err),
+    }
+}
+
+unsafe extern "C" fn info_fn(msg: *const libc::c_char) {
+    match std::ffi::CStr::from_ptr(msg).to_str() {
+        Ok(msg) => log::info!("{}", msg),
+        Err(err) => log::error!("failed to parse JACK error: {:?}", err),
+    }
+}
